
#include <stdbool.h>
#include <stdint.h>
#include <inttypes.h>
#include "nrf_delay.h"
#include "nrf_gpio.h"
#include "bsp.h"
#include "boards.h"
#include "nrf.h"
#include "app_uart.h"

#define GSM_RX_PIN_NUMBER 7 	//7
#define GSM_TX_PIN_NUMBER 6		//6
#define GSM_RTS_PIN_NUMBER 5	//5
#define GSM_CTS_PIN_NUMBER 11	//Pin 8 is probably broken
#define GSM_UART_FLOW_CONTROL APP_UART_FLOW_CONTROL_ENABLED
#define GSM_USE_PARITY false
#define GSM_UART_BAUDRATE UART_BAUDRATE_BAUDRATE_Baud9600
#define GSM_UART_TX_BUF_SIZE 128u
#define GSM_UART_RX_BUF_SIZE 128u
#define GSM_UART_PRIORITY APP_IRQ_PRIORITY_HIGH

#define LED_RX LED_1
#define LED_FRAME_ERROR LED_2
#define LED_OVERRUN_ERROR LED_3
#define LED_BREAK_ERROR LED_4

void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
	NVIC_SystemReset();
}


void gsm_uart_evt_callback(app_uart_evt_t * uart_evt)
{
	switch (uart_evt->evt_type)
	{
	case APP_UART_DATA:
		break;
	case APP_UART_DATA_READY:
	{
		uint8_t cr;
		uint32_t error;

		//Flash the led to show that rx has been received
		nrf_gpio_pin_clear(LED_RX);

		error = app_uart_get(&cr);
		APP_ERROR_CHECK(error);

		nrf_gpio_pin_set(LED_RX);
		break;
	}
	case APP_UART_TX_EMPTY:
		break;
	case APP_UART_COMMUNICATION_ERROR:
	{
		uint32_t error = uart_evt->data.error_communication;

		if ((error & 1) != 0) //Overrun error
		{
			nrf_gpio_pin_clear(LED_OVERRUN_ERROR);
			nrf_delay_ms(10);
			nrf_gpio_pin_set(LED_OVERRUN_ERROR);
		}
		if ((error & 4) != 0) //Frame error
		{
			nrf_gpio_pin_clear(LED_FRAME_ERROR);
			nrf_delay_ms(10);
			nrf_gpio_pin_set(LED_FRAME_ERROR);
		}
		if ((error & 8) != 0) //Break error
		{
			nrf_gpio_pin_clear(LED_OVERRUN_ERROR);
			nrf_delay_ms(10);
			nrf_gpio_pin_set(LED_OVERRUN_ERROR);
		}

		//APP_ERROR_CHECK(error);

	    break;
	}
	case APP_UART_FIFO_ERROR:
	    APP_ERROR_HANDLER(uart_evt->data.error_code);
	    break;
	default:
		break;
	}
}

/**
 * @brief Function for application main entry.
 */
int main(void)
{
	nrf_gpio_cfg_output(LED_RX);
	nrf_gpio_cfg_output(LED_FRAME_ERROR);
	nrf_gpio_cfg_output(LED_OVERRUN_ERROR);
	nrf_gpio_cfg_output(LED_BREAK_ERROR);
	nrf_gpio_pin_set(LED_RX);
	nrf_gpio_pin_set(LED_FRAME_ERROR);
	nrf_gpio_pin_set(LED_OVERRUN_ERROR);
	nrf_gpio_pin_set(LED_BREAK_ERROR);


	uint32_t err_code;

	static const app_uart_comm_params_t comm_params =
	    {
	          GSM_RX_PIN_NUMBER,
	          GSM_TX_PIN_NUMBER,
	          GSM_RTS_PIN_NUMBER,
	          GSM_CTS_PIN_NUMBER,
			  GSM_UART_FLOW_CONTROL,
	          GSM_USE_PARITY,
			  GSM_UART_BAUDRATE
	    };


    APP_UART_FIFO_INIT(&comm_params,
                         GSM_UART_RX_BUF_SIZE,
                         GSM_UART_TX_BUF_SIZE,
                         gsm_uart_evt_callback,
                         GSM_UART_PRIORITY,
                         err_code);

    APP_ERROR_CHECK(err_code);

    while (true)
    {
    	while (app_uart_put('A') != NRF_SUCCESS);
    	while (app_uart_put('T') != NRF_SUCCESS);
    	while (app_uart_put('\r') != NRF_SUCCESS);

    	nrf_delay_ms(2000);
    }
}


/** @} */
